@@ -53,7 +53,6 @@ static void ccfc_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	int i;
 	int maxlen;
 
-	dump_response("ccfc_query_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok)
@@ -112,10 +111,10 @@ static void ccfc_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	}
 
 	for (i = 0; i < num; i++)
-		ofono_debug("ccfc_cb: %d, %d, %s(%d) - %d sec",
-				list[i].status, list[i].cls,
-				list[i].phone_number.number,
-				list[i].phone_number.type, list[i].time);
+		DBG("ccfc_cb: %d, %d, %s(%d) - %d sec",
+			list[i].status, list[i].cls,
+			list[i].phone_number.number,
+			list[i].phone_number.type, list[i].time);
 
 out:
 	cb(&error, num, list, cbd->data);
@@ -129,23 +128,18 @@ static void at_ccfc_query(struct ofono_call_forwarding *cf, int type, int cls,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[64];
 
-	if (!cbd)
-		goto error;
-
 	cbd->user = GINT_TO_POINTER(cls);
 
 	if (cls == 7)
-		sprintf(buf, "AT+CCFC=%d,2", type);
+		snprintf(buf, sizeof(buf), "AT+CCFC=%d,2", type);
 	else
-		sprintf(buf, "AT+CCFC=%d,2,,,%d", type, cls);
+		snprintf(buf, sizeof(buf), "AT+CCFC=%d,2,,,%d", type, cls);
 
 	if (g_at_chat_send(chat, buf, ccfc_prefix,
 				ccfc_query_cb, cbd, g_free) > 0)
 		return;
 
-error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, 0, NULL, data);
 }
@@ -156,7 +150,6 @@ static void ccfc_set_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	ofono_call_forwarding_set_cb_t cb = cbd->cb;
 	struct ofono_error error;
 
-	dump_response("ccfc_set_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	cb(&error, cbd->data);
@@ -168,16 +161,11 @@ static void at_ccfc_set(struct ofono_call_forwarding *cf, const char *buf,
 	GAtChat *chat = ofono_call_forwarding_get_data(cf);
 	struct cb_data *cbd = cb_data_new(cb, data);
 
-	if (!cbd)
-		goto error;
-
 	if (g_at_chat_send(chat, buf, none_prefix,
 				ccfc_set_cb, cbd, g_free) > 0)
 		return;
 
-error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
 }
@@ -189,10 +177,10 @@ static void at_ccfc_erasure(struct ofono_call_forwarding *cf,
 	char buf[128];
 	int len;
 
-	len = sprintf(buf, "AT+CCFC=%d,4", type);
+	len = snprintf(buf, sizeof(buf), "AT+CCFC=%d,4", type);
 
 	if (cls != 7)
-		sprintf(buf + len, ",,,%d", cls);
+		snprintf(buf + len, sizeof(buf) - len, ",,,%d", cls);
 
 	at_ccfc_set(cf, buf, cb, data);
 }
@@ -205,10 +193,10 @@ static void at_ccfc_deactivation(struct ofono_call_forwarding *cf,
 	char buf[128];
 	int len;
 
-	len = sprintf(buf, "AT+CCFC=%d,0", type);
+	len = snprintf(buf, sizeof(buf), "AT+CCFC=%d,0", type);
 
 	if (cls != 7)
-		sprintf(buf + len, ",,,%d", cls);
+		snprintf(buf + len, sizeof(buf) - len, ",,,%d", cls);
 
 	at_ccfc_set(cf, buf, cb, data);
 }
@@ -220,10 +208,10 @@ static void at_ccfc_activation(struct ofono_call_forwarding *cf,
 	char buf[128];
 	int len;
 
-	len = sprintf(buf, "AT+CCFC=%d,1", type);
+	len = snprintf(buf, sizeof(buf), "AT+CCFC=%d,1", type);
 
 	if (cls != 7)
-		sprintf(buf + len, ",,,%d", cls);
+		snprintf(buf + len, sizeof(buf) - len, ",,,%d", cls);
 
 	at_ccfc_set(cf, buf, cb, data);
 }
@@ -238,11 +226,11 @@ static void at_ccfc_registration(struct ofono_call_forwarding *cf,
 	char buf[128];
 	int offset;
 
-	offset = sprintf(buf, "AT+CCFC=%d,3,\"%s\",%d,%d", type,
+	offset = snprintf(buf, sizeof(buf), "AT+CCFC=%d,3,\"%s\",%d,%d", type,
 				ph->number, ph->type, cls);
 
 	if (type == 2 || type == 4 || type == 5)
-		sprintf(buf+offset, ",,,%d", time);
+		snprintf(buf+offset, sizeof(buf) - offset, ",,,%d", time);
 
 	at_ccfc_set(cf, buf, cb, data);
 }
@@ -261,7 +249,7 @@ static int at_ccfc_probe(struct ofono_call_forwarding *cf, unsigned int vendor,
 {
 	GAtChat *chat = data;
 
-	ofono_call_forwarding_set_data(cf, chat);
+	ofono_call_forwarding_set_data(cf, g_at_chat_clone(chat));
 	g_idle_add(at_ccfc_register, cf);
 
 	return 0;
@@ -269,6 +257,10 @@ static int at_ccfc_probe(struct ofono_call_forwarding *cf, unsigned int vendor,
 
 static void at_ccfc_remove(struct ofono_call_forwarding *cf)
 {
+	GAtChat *chat = ofono_call_forwarding_get_data(cf);
+
+	g_at_chat_unref(chat);
+	ofono_call_forwarding_set_data(cf, NULL);
 }
 
 static struct ofono_call_forwarding_driver driver = {
@@ -282,12 +274,12 @@ static struct ofono_call_forwarding_driver driver = {
 	.erasure	= at_ccfc_erasure
 };
 
-void at_call_forwarding_init()
+void at_call_forwarding_init(void)
 {
 	ofono_call_forwarding_driver_register(&driver);
 }
 
-void at_call_forwarding_exit()
+void at_call_forwarding_exit(void)
 {
 	ofono_call_forwarding_driver_unregister(&driver);
 }
