@@ -42,11 +42,12 @@
 #include <ofono/ussd.h>
 #include <ofono/gprs.h>
 #include <ofono/gprs-context.h>
+#include <ofono/audio-settings.h>
+#include <ofono/radio-settings.h>
 #include <ofono/voicecall.h>
 #include <ofono/call-forwarding.h>
 #include <ofono/call-settings.h>
 #include <ofono/call-barring.h>
-#include <ofono/ssn.h>
 #include <ofono/phonebook.h>
 #include <ofono/message-waiting.h>
 #include <ofono/log.h>
@@ -76,6 +77,7 @@ struct huawei_data {
 	struct ofono_gprs *gprs;
 	struct ofono_gprs_context *gc;
 	gboolean voice;
+	gboolean ndis;
 	guint sim_poll_timeout;
 	guint sim_poll_count;
 };
@@ -91,7 +93,7 @@ static int huawei_probe(struct ofono_modem *modem)
 	DBG("%p", modem);
 
 	data = g_try_new0(struct huawei_data, 1);
-	if (!data)
+	if (data == NULL)
 		return -ENOMEM;
 
 	ofono_modem_set_data(modem, data);
@@ -107,7 +109,9 @@ static void huawei_remove(struct ofono_modem *modem)
 
 	ofono_modem_set_data(modem, NULL);
 
-	g_at_chat_unref(data->modem);
+	if (data->modem)
+		g_at_chat_unref(data->modem);
+
 	g_at_chat_unref(data->pcui);
 	g_free(data);
 }
@@ -201,6 +205,11 @@ static gboolean notify_sim_state(struct ofono_modem *modem,
 	case HUAWEI_SIM_STATE_INVALID_CS:
 	case HUAWEI_SIM_STATE_INVALID_PS:
 	case HUAWEI_SIM_STATE_INVALID_PS_AND_CS:
+		if (data->sim_poll_timeout) {
+			g_source_remove(data->sim_poll_timeout);
+			data->sim_poll_timeout = 0;
+		}
+
 		/*
 		 * In the "warm start" case the modem skips
 		 * HUAWEI_SIM_STATE_INVALID_OR_LOCKED altogether, so need
@@ -208,7 +217,8 @@ static gboolean notify_sim_state(struct ofono_modem *modem,
 		 */
 		ofono_modem_set_powered(modem, TRUE);
 
-		g_at_chat_send(data->pcui, "AT+CFUN=5", none_prefix,
+		if (ofono_modem_get_online(modem) == FALSE)
+			g_at_chat_send(data->pcui, "AT+CFUN=5", none_prefix,
 				cfun_offline, modem, NULL);
 
 		return FALSE;
@@ -217,6 +227,30 @@ static gboolean notify_sim_state(struct ofono_modem *modem,
 	return FALSE;
 }
 
+static void cpin_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct ofono_modem *modem = user_data;
+
+	if (!ok)
+		return;
+
+	/* Force notification of SIM ready because it's in a locked state */
+	notify_sim_state(modem, HUAWEI_SIM_STATE_VALID);
+}
+
+static gboolean query_sim_locked(gpointer user_data)
+{
+	struct ofono_modem *modem = user_data;
+	struct huawei_data *data = ofono_modem_get_data(modem);
+
+	data->sim_poll_timeout = 0;
+
+	g_at_chat_send(data->pcui, "AT+CPIN?", NULL,
+			cpin_cb, modem, NULL);
+
+	return FALSE;
+}
+
 static void sysinfo_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct ofono_modem *modem = user_data;
@@ -255,6 +289,11 @@ static void sysinfo_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		data->sim_poll_timeout = g_timeout_add_seconds(2,
 								query_sim_state,
 								modem);
+	} else if (sim_state ==	HUAWEI_SIM_STATE_INVALID_OR_LOCKED &&
+						!data->sim_poll_timeout) {
+		data->sim_poll_timeout = g_timeout_add_seconds(2,
+								query_sim_locked,
+								modem);
 	}
 }
 
@@ -388,7 +427,7 @@ static GAtChat *create_port(const char *device)
 	GAtChat *chat;
 
 	channel = g_at_tty_open(device, NULL);
-	if (!channel)
+	if (channel == NULL)
 		return NULL;
 
 	syntax = g_at_syntax_new_gsm_permissive();
@@ -396,7 +435,7 @@ static GAtChat *create_port(const char *device)
 	g_at_syntax_unref(syntax);
 	g_io_channel_unref(channel);
 
-	if (!chat)
+	if (chat == NULL)
 		return NULL;
 
 	return chat;
@@ -431,10 +470,7 @@ static void huawei_disconnect(gpointer user_data)
 	struct ofono_modem *modem = user_data;
 	struct huawei_data *data = ofono_modem_get_data(modem);
 
-	DBG("");
-
-	if (data->gc)
-		ofono_gprs_context_remove(data->gc);
+	DBG("data->gc %p", data->gc);
 
 	g_at_chat_unref(data->modem);
 	data->modem = NULL;
@@ -446,6 +482,12 @@ static void huawei_disconnect(gpointer user_data)
 	g_at_chat_set_disconnect_function(data->modem,
 						huawei_disconnect, modem);
 
+	/* gprs_context has been destructed and needs not reopen */
+	if (data->gc == NULL)
+		return;
+
+	ofono_gprs_context_remove(data->gc);
+
 	if (data->sim_state == HUAWEI_SIM_STATE_VALID ||
 			data->sim_state == HUAWEI_SIM_STATE_INVALID_CS) {
 		ofono_info("Reopened GPRS context channel");
@@ -464,12 +506,15 @@ static int huawei_enable(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
-	data->modem = open_device(modem, "Modem", "Modem: ");
-	if (data->modem == NULL)
-		return -EINVAL;
+	if (ofono_modem_get_string(modem, "NDIS") == NULL) {
+		data->modem = open_device(modem, "Modem", "Modem: ");
+		if (data->modem == NULL)
+			return -EINVAL;
 
-	g_at_chat_set_disconnect_function(data->modem,
+		g_at_chat_set_disconnect_function(data->modem,
 						huawei_disconnect, modem);
+	} else
+		data->ndis = TRUE;
 
 	data->pcui = open_device(modem, "Pcui", "PCUI: ");
 	if (data->pcui == NULL) {
@@ -483,7 +528,8 @@ static int huawei_enable(struct ofono_modem *modem)
 
 	data->sim_state = 0;
 
-	g_at_chat_send(data->pcui, "ATE0", none_prefix, NULL, NULL, NULL);
+	g_at_chat_send(data->pcui, "ATE0 +CMEE=1", none_prefix,
+						NULL, NULL, NULL);
 
 	g_at_chat_send(data->pcui, "AT+CFUN=1", none_prefix,
 					cfun_enable, modem, NULL);
@@ -525,7 +571,7 @@ static int huawei_disable(struct ofono_modem *modem)
 		data->modem = NULL;
 	}
 
-	if (!data->pcui)
+	if (data->pcui == NULL)
 		return 0;
 
 	g_at_chat_cancel_all(data->pcui);
@@ -540,11 +586,27 @@ static void set_online_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
 	ofono_modem_online_cb_t cb = cbd->cb;
+	struct ofono_error error;
 
-	if (ok)
-		CALLBACK_WITH_SUCCESS(cb, cbd->data);
-	else
-		CALLBACK_WITH_FAILURE(cb, cbd->data);
+	decode_at_error(&error, g_at_result_final_response(result));
+	cb(&error, cbd->data);
+}
+
+static void set_offline_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	ofono_modem_online_cb_t cb = cbd->cb;
+	struct ofono_error error;
+
+	if (ok) {
+		struct huawei_data *data = cbd->user;
+
+		data->gc = NULL;
+		data->gprs = NULL;
+	}
+
+	decode_at_error(&error, g_at_result_final_response(result));
+	cb(&error, cbd->data);
 }
 
 static void huawei_set_online(struct ofono_modem *modem, ofono_bool_t online,
@@ -557,13 +619,13 @@ static void huawei_set_online(struct ofono_modem *modem, ofono_bool_t online,
 
 	DBG("modem %p %s", modem, online ? "online" : "offline");
 
-	if (!cbd)
-		goto error;
+	cbd->user = data;
 
-	if (g_at_chat_send(chat, command, NULL, set_online_cb, cbd, g_free))
+	if (g_at_chat_send(chat, command, NULL,
+				online ? set_online_cb : set_offline_cb,
+				cbd, g_free))
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, cbd->data);
@@ -576,11 +638,8 @@ static void huawei_pre_sim(struct ofono_modem *modem)
 	DBG("%p", modem);
 
 	ofono_devinfo_create(modem, 0, "atmodem", data->pcui);
-	data->sim = ofono_sim_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
+	data->sim = ofono_sim_create(modem, OFONO_VENDOR_HUAWEI,
 					"atmodem", data->pcui);
-
-	data->sim_poll_count = 0;
-	query_sim_state(modem);
 }
 
 static void huawei_post_sim(struct ofono_modem *modem)
@@ -589,11 +648,16 @@ static void huawei_post_sim(struct ofono_modem *modem)
 
 	DBG("%p", modem);
 
-	if (data->voice == TRUE)
-		ofono_voicecall_create(modem, OFONO_VENDOR_HUAWEI,
+	if (data->voice == TRUE) {
+		ofono_voicecall_create(modem, 0, "huaweimodem", data->pcui);
+		ofono_audio_settings_create(modem, 0,
 						"huaweimodem", data->pcui);
+	}
 
 	ofono_phonebook_create(modem, 0, "atmodem", data->pcui);
+	ofono_radio_settings_create(modem, 0, "huaweimodem", data->pcui);
+
+	ofono_sms_create(modem, OFONO_VENDOR_HUAWEI, "atmodem", data->pcui);
 }
 
 static void huawei_post_online(struct ofono_modem *modem)
@@ -603,8 +667,8 @@ static void huawei_post_online(struct ofono_modem *modem)
 	struct ofono_message_waiting *mw;
 
 	if (data->sim_state != HUAWEI_SIM_STATE_VALID &&
-	    data->sim_state != HUAWEI_SIM_STATE_INVALID_CS &&
-	    data->sim_state != HUAWEI_SIM_STATE_INVALID_PS) {
+			data->sim_state != HUAWEI_SIM_STATE_INVALID_CS &&
+			data->sim_state != HUAWEI_SIM_STATE_INVALID_PS) {
 		ofono_info("huawei: invalid sim state in post online (%d)",
 				data->sim_state);
 		return;
@@ -613,17 +677,23 @@ static void huawei_post_online(struct ofono_modem *modem)
 	netreg = ofono_netreg_create(modem, OFONO_VENDOR_HUAWEI, "atmodem",
 								data->pcui);
 
-	ofono_sms_create(modem, OFONO_VENDOR_HUAWEI, "atmodem", data->pcui);
 	ofono_cbs_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
 						"atmodem", data->pcui);
 	ofono_ussd_create(modem, OFONO_VENDOR_QUALCOMM_MSM,
 						"atmodem", data->pcui);
 
-	if (data->sim_state == HUAWEI_SIM_STATE_VALID ||
-			data->sim_state == HUAWEI_SIM_STATE_INVALID_CS) {
-		data->gprs = ofono_gprs_create(modem, 0, "atmodem", data->pcui);
-		data->gc = ofono_gprs_context_create(modem, 0, "atmodem",
-								data->modem);
+	if ((data->sim_state == HUAWEI_SIM_STATE_VALID ||
+			data->sim_state == HUAWEI_SIM_STATE_INVALID_CS) &&
+			data->modem != NULL) {
+		data->gprs = ofono_gprs_create(modem, OFONO_VENDOR_HUAWEI,
+						"atmodem", data->pcui);
+
+		if (data->ndis == TRUE)
+			data->gc = ofono_gprs_context_create(modem, 0,
+						"huaweimodem", data->pcui);
+		else
+			data->gc = ofono_gprs_context_create(modem, 0,
+						"atmodem", data->modem);
 
 		if (data->gprs && data->gc)
 			ofono_gprs_add_context(data->gprs, data->gc);
@@ -635,7 +705,6 @@ static void huawei_post_online(struct ofono_modem *modem)
 		ofono_call_forwarding_create(modem, 0, "atmodem", data->pcui);
 		ofono_call_settings_create(modem, 0, "atmodem", data->pcui);
 		ofono_call_barring_create(modem, 0, "atmodem", data->pcui);
-		ofono_ssn_create(modem, 0, "atmodem", data->pcui);
 
 		mw = ofono_message_waiting_create(modem);
 		if (mw)
